<?php

/**
 * Add entry in admin menu.
 */
function zcmail_add_pages() {
	$page = add_options_page(__('Zero Conf Mail', 'zero-conf-mail'), __('Zero Conf Mail', 'zero-conf-mail'), 10, 'zero-conf-mail', 'zcmail_options_page');
	add_action( 'admin_head-' . $page, 'zcmail_css_admin' );
}

/**
 * Load admin CSS style
 *
 * @since 0.1.10
 *
 * @todo check if this is correct
 */
function zcmail_css_admin() { ?>
	<link rel="stylesheet" href="<?php echo get_bloginfo( 'home' ) . '/' . PLUGINDIR . '/zero-conf-mail/inc/admin.css' ?>" type="text/css" media="all" /> <?php
}

/**
 * Plugin activation hook
 */
function zcmail_activate() {
	if ( !get_option('zcmail') ) {
		// fresh install
		zcmail_reset();
		return;
	}

	$old_option=get_option('zcmail');

	// migrate to 0.1.0 data structure
	if ( version_compare( $old_option['version'] , '0.1.0') == -1 ) {
		zcmail_reset();
		$new_option=get_option('zcmail');

		$new_option['config']['to'] = $old_option['recipient'];
		$new_option['config']['js'] = 'Enabled';

		$new_option['fields']['name']['label'] = $old_option['name'];
		$new_option['fields']['subject']['label'] = $old_option['subject'];
		$new_option['fields']['from']['label'] = $old_option['mail'];
		$new_option['fields']['body']['label'] = $old_option['message'];
		$new_option['fields']['submit']['label'] = $old_option['submit'];

		update_option('zcmail', $new_option);
	}
}

/**
 * Clean up when uninstalled, delete settings from db
 */
function zcmail_uninstall() {
	delete_option('zcmail');
}

/**
 * Reset the configuration. Used for default configuration as well.
 * Get sane, localized defaults and get current user's email address.
 *
 * @since 0.1.0
 */
function zcmail_reset() {
	global $user_email;
	$default = array(
		'version'	=> '0.1.0',
		// currently used for new fields
		'blueprints' => array(
			'new' => array(
				'label' => __('Your new field', 'zero-conf-mail'),
				'type'	=> 'text',
				'must'	=> false,
				'pos'	=> 0,
			),
		),
		'fields'	=> array(
			'name'	=> array(
				'pos'	=> 10,
				'label' => __('Your name', 'zero-conf-mail'),
				'type'	=> 'text',
				'must'	=> true,
			),
			'from'	=> array(
				'pos'	=> 20,
				'label' => __('Your email', 'zero-conf-mail'),
				'type'	=> 'email',
				'must'	=> true,
			),
			'subject'	=> array(
				'pos'	=> 30,
				'label' => __('Subject', 'zero-conf-mail'),
				'type'	=> 'text',
				'must'	=> false,
			),
			'body'		=> array(
				'pos'	=> 40,
				'label' => __('Your message', 'zero-conf-mail'),
				'type'	=> 'textarea',
				'must'	=> true,
			),
			'submit'	=> array(
				'pos'	=> 50,
				'label' => __('Send message', 'zero-conf-mail'),
				'type'	=> 'submit',
			),
		),
		// FIXME config shouldn't be localized, breaks on language switch
		'config'	=> array(
			'to'	=> $user_email,
			'js'		=> 'Enabled',
			'homelink'	=> 'In the footer',
			'messages'	=> array(
				'failure'	=> __('Please fill out all fields!', 'zero-conf-mail'),
				'success'	=> __('Your message has been sent. Thank you!', 'zero-conf-mail'),
			),
			'input_types' => array(
				'text',
				'textarea',
				'email',
				'submit',
			),
			'advanced' => 'Disabled',
			'homelink_choices' => array(
				'Below the form',
				'In the footer',
				'Nowhere',
			),
			'enable_choices' => array(
				'Enabled',
				'Disabled',
			),
		),
	);
	$default = zcmail_uasort($default);
	update_option('zcmail', $default );
}

/**
 * Sort fields by position
 *
 * @param array $data multiple fields
 * @return array multiple sorted fields
 */
function zcmail_uasort($data) {
	uasort($data['fields'], 'zcmail_uasort_callback');
	return($data);
}

/**
 * Callback for uasort, see http://php.net/manual/function.uasort.php
 *
 * @param array $a single field
 * @param array $b single field
 *
 * @return array sorted fields
 */
function zcmail_uasort_callback($a, $b) {
	return ($a['pos'] < $b['pos'] ) ? -1 : 1;
}

/**
 * Print the options page
 *
 * @todo make this more readable...
 * @todo check if we should encode html entities in field labels
 */
function zcmail_options_page() {
	// Check permissions
	if (current_user_can('manage_options')) { ?>
		<div id="nkuttler" > <?php
		if ($_POST['pass']) {
			function_exists('check_admin_referer') ? check_admin_referer('zero-conf-mail') : null;
		}
		$option = get_option('zcmail');
		//$data = $_POST;
		// TODO update codex nonce docs.....
		unset($_POST['_wp_http_referer']);
		unset($_POST['_wpnonce']);
		switch ( $_POST['pass'] ) {
			case 'admin_fields':
				$updated_fields = $_POST;

				unset($updated_fields['pass']);

				// merge new into old values
				foreach ($option['fields'] as $key => $field) {
					$option['fields'][$key] = array_merge($field, $updated_fields[$key]);
				}

				// strip slashes from labels
				foreach ( $option['fields'] as $fieldname => $values ) {
					$option['fields'][$fieldname]['label'] = stripslashes($values['label']);
				}
				
				// sort the result
				$option = zcmail_uasort($option);
				update_option('zcmail', $option);
				break;
			case 'config':
				$config = $_POST['config'];
				// TODO this won't work for arrays...
				foreach($config as $key => $value) {
					$option['config'][$key] = $value;
				}
				update_option('zcmail', $option);
				break;
			case 'reset':
				zcmail_reset();
				break;
			case 'add':
				$new = $_POST['new'];
				$option['fields'][] = $new;
				$option = zcmail_uasort($option);
				update_option('zcmail', $option);
				_e('I added a field', 'zero-conf-mail');
				break;
		}

		if ( !is_email( $option['config']['to'] ) ) { ?>
			<div class="warn box"><?php _e("It looks like the configured email isn't valid, please check the advanced configuration", 'zero-conf-mail' ) ?></div> <?php
		} ?>

		<h2><?php _e('Zero Conf Mail', 'zero-conf-mail' )?></h2>
		<?php zcmail_links( 'zero-conf-mail' ) ?>
		<p> <?php _e("I'm sure I am reinventing the wheel with this plugin. However, sometimes it's faster to simply write something yourself than it is to find something that does what you want. This plugin uses no Ajax, no bloated libraries, no external files and doesn't need to be configured at all. Just include it on a page by putting <tt>[zcmail]</tt> into the page content." ,'zero-conf-mail' )?> </p>

		<h2><?php _e('Current Configuration', 'zero-conf-mail')?></h2>
		<p> <strong><?php _e('Important!', 'zero-conf-mail') ?></strong> <?php _e('You are not supposed to put your own name etc. in here. These fields are used as labels for your mailform. However, feel free to customize the settings.', 'zero-conf-mail') ?> </p> <?php
		$option = get_option('zcmail'); ?>
		<form action="" method="post">
			<?php function_exists('wp_nonce_field') ? wp_nonce_field('zero-conf-mail') : null; ?>
			<table >
				<?php zcmail_thead($option['config']['advanced'] ) ?>
				<tbody><?php
					zcmail_admin_rows( $option['fields'], $option['config'] );
				?>
				</tbody>
			</table>
			<br />
			<input type="hidden" name="pass" value="admin_fields" />
			<input type="submit" class="button-primary" value="<?php _e('Save Changes', 'zero-conf-mail') ?>" />

			<!--
			<h3><?php _e('Add a pinch of JavaScript?', 'zero-conf-mail')?></h3>
			<p>
				<?php _e('I recommend that you enable this. It will just add a few characters to the HTML source of the contact form. It doesn\'t depend on any external JavaScript.', 'zero-conf-mail')?>
			</p>
			<?php zcmail_select('config_js', $option['config']['js'], $option['config']['enable_choices'] ); ?>
			<input type="submit" class="button-primary" value="<?php _e('Save Changes', 'zero-conf-mail') ?>">
			-->
		</form>

		<h2><?php _e('Advanced settings', 'zero-conf-mail') ?></h2>
		<form action="" method="post">
			<?php function_exists('wp_nonce_field') ? wp_nonce_field('zero-conf-mail') : null; ?>
			<input type="hidden" name="pass" value="config" />
			<?php _e('Show the advanced options?', 'zero-conf-mail' ) ?>
			<?php zcmail_select("config[advanced]", $option['config']['advanced'], $option['config']['enable_choices'] ); ?>
			<input type="submit" class="button-secondary" value="<?php _e('Save Changes', 'zero-conf-mail') ?>" />
		</form> <?php

		if ($option['config']['advanced'] == 'Enabled') { ?>
			<form action="" method="post">
				<?php function_exists('wp_nonce_field') ? wp_nonce_field('zero-conf-mail') : null; ?>
				<input type="hidden" name="pass" value="config" />
				<?php _e('Who receives the emails', 'zero-conf-mail' ) ?>
				<?php zcmail_input("config[to]", $option['config']['to'], false ); ?>
				<br />

				<?php _e('Where to put the home link', 'zero-conf-mail' ) ?>
				<?php zcmail_select("config[homelink]", $option['config']['homelink'], $option['config']['homelink_choices'] ) ?>
				<br />

				<?php _e("Message when the form wasn't completed", 'zero-conf-mail' ) ?>
				<?php zcmail_input("config[messages][failure]", $option['config']['messages']['failure'], false ); ?>
				<br />

				<?php _e("Message when the mail was sent", 'zero-conf-mail' ) ?>
				<?php zcmail_input("config[messages][success]", $option['config']['messages']['success'], false ); ?>
				<br />

				<input type="submit" class="button-secondary" value="<?php _e('Save Changes', 'zero-conf-mail') ?>" />
			</form> 

			<h2><?php _e('Add a custom field', 'zero-conf-mail') ?></h2>
			<form action="" method="post">
				<?php function_exists('wp_nonce_field') ? wp_nonce_field('zero-conf-mail') : null; ?>
				<input type="hidden" name="pass" value="add" />
				<table > <?php
				zcmail_thead($option['config']['advanced'] );
				zcmail_admin_rows( $option['blueprints'], $option['config'] ); ?>
				</table>
				<input type="submit" class="button-secondary" value="<?php _e('Add Field', 'zero-conf-mail') ?>" />
			</form>
			<div class="warn box " >
				<h3><?php _e('Reset the form', 'zero-conf-mail') ?></h3>
				<p><?php _e('<strong>Careful!</strong> This will delete any customizations you have made!') ?></p>
				<form action="" method="post">
					<?php function_exists('wp_nonce_field') ? wp_nonce_field('zero-conf-mail') : null; ?>
					<input type="hidden" name="pass" value="reset" />
					<input type="submit" class="button-secondary" value="<?php _e('Reset the form', 'zero-conf-mail') ?>" />
				</form>
			</div> <?php
		} ?>
	</div>
	<?php
	}
}

/**
 * Print rows from a set of field definitions
 *
 * @param array $data field definitions
 * @param array $config configuration definitions
 */
function zcmail_admin_rows($data, $config ) {
	foreach ( $data as $name => $field ) {
	?>
		<tr>
			<td>
				<?php zcmail_input($name."[label]", $field['label'], false ); ?>
			</td> <?php
			if ($config['advanced'] == 'Enabled' ) { ?>
				<td>
					<?php zcmail_select($name."[type]", $field['type'], $config['input_types'] ) ?>
				</td>
				<td >
					<?php zcmail_bool($name."[must]", $field['must']) ?>
				</td>
				<td>
					<?php zcmail_input($name."[pos]", $field['pos'], false, 3 ); ?>
				</td> <?php
			} ?>
		</tr><?php
	}
}

/**
 * The admin fields list table header
 *
 * @param string $advanced show advanced columns
 */
function zcmail_thead($advanced) { ?>
	<thead> 
		<tr>
			<td><?php _e('Label', 'zero-conf-mail') ?></td><?php
			if ($advanced == 'Enabled') { ?>
				<td><?php _e('Type', 'zero-conf-mail') ?></td>
				<td><?php _e('Mandatory (y/n)?', 'zero-conf-mail') ?></td>
				<td><?php _e('Position', 'zero-conf-mail') ?></td> <?php
			} ?>
		</tr>
	</thead><?php
}
?>
