<?php

/**
 * Get post data and do and send the mail
 *
 * @todo Hardcoded field names. Current fix is to disallow field deletion.
 */
function zcmail_post2mail() { ?>
	<div id="zcmail"><?php
	$option = get_option('zcmail');
	if ( isset($_POST['zcmail'] )) {
		if ( zcmail_validates($_POST['zcmail'], $option['fields'] ) ) {
			if ( $_POST['zcmail']['from'] ) {
				$from = $_POST['zcmail']['from'];
			}
			else {
				$from = 'Bug 01 - no from field defined';
			}

			if ( $_POST['zcmail']['subject'] ) {
				$subject = $_POST['zcmail']['subject'];
			}
			else {
				$subject = 'Bug 02 - no subject field defined';
			}

			if ( $_POST['zcmail']['name'] ) {
				$message = $_POST['zcmail']['name'];
			}

			if ( $_POST['zcmail']['body'] ) {
				$message .= "\n\n" . stripslashes( $_POST['zcmail']['body'] );
			}
			else {
				$message .= "\n\n" . 'Bug 03 - no message field defined';
			}

			foreach($_POST['zcmail'] as $field => $values) {
				// TODO I guess this should be safe, as a[] = foo => a[0] == foo 
				// Add custom fields to the message
				if ( is_integer( $field ) ) {
					$message .= "\n";
					$message .= $option['fields'][$field]['label'] . ": $values";
				}
			}

			// Add message footer
			$message .= "\n\n---\n" . __('Sent with Zero Conf Mail: ', 'zero-conf-mail');
			$message .= __('http://www.nkuttler.de/wordpress/zero-conf-mail/', 'zero-conf-mail');
			$message .= "\n\n" . __('Blog: ', 'zero-conf-mail');
			$message .= get_bloginfo('home');
			$message .= "\n" . __('IP  : ', 'zero-conf-mail');
			$message .= $_SERVER['REMOTE_ADDR'];

			$message = zcmail_sanitize( $message );

			$success = zcmail_sendmail( $option['config']['to'], $from, $subject, $message );

			if ( $success ) {
				echo '<div class="zcmail_success">';
				echo $option['config']['messages']['success'];
				echo '</div>';
			}
			else {
				echo '<div style="background-color:#f99;color:black;font-weight:bold;font-size:larger;padding:10px;"> ';
				_e( "No message could be sent. This means that this server wasn't configured to send mail. Please contact the site administrator.", 'zero-conf-mail' );
				echo '</div>';
			}
		}
		else {
			echo '<div class="zcmail_error">';
			echo $option['config']['messages']['failure'];
			echo '</div>';
			zcmail_printform();
		}
	}
	else {
		zcmail_printform();
	} ?>
	</div> <?php
}

/**
 * Send a mail
 * 
 * @param string $to receiver
 * @param string $from sender
 * @param string $subject subject
 * @param string $body email body
 *
 * @return was the message accepted for delivery
 */
function zcmail_sendmail($to, $from, $subject, $body) {
	$result = mail(
		$to,
		'[' . get_bloginfo('name') . '] ' . $subject,
		$body,
		"From: $from"
	);
	return $result;
}

/**
 * Perform (basic) sanitization of user input.
 *
 * @param $string string to sanitize
 * @return string sanitized string
 */
function zcmail_sanitize( $string ) {
	$string = htmlspecialchars($string, ENT_NOQUOTES);;
	return $string;
}

/**
 * Validate all parameters
 *
 * @param array $input $_POST data
 * @param array $defs configuration options
 * 
 * @return boolean do all fields validate
 *
 * @todo Useful error messages per field
 */
function zcmail_validates($input, $defs) {
	foreach($defs as $def => $value) {
		if ($value['type'] == 'text' || $value['type'] == 'textarea') {
			// catch all mandatory fields that haven't been modified
			if ( $value['must'] == 'Enabled' && $input[$def] == $value['label'] ) {
				//echo "error: " . $input[$def] . " unchanged";
				return false;
			}
			// catch all mandatory fields that are empty
			elseif ( $value['must'] == 'Enabled' && $input[$def] == '' ) {
				//echo "error: " . $value['label'] . $input[$def] . " empty";
				return false;
			}
		}
		if ($value['type'] == 'email') {
			// catch all mandatory invalid mail addresses
			if ( $value['must'] == 'Enabled' && $value['type'] == 'email' && !is_email($input[$def]) ) {
				//echo "error: " . $input[$def] . " no mail";
				return false;
			}
		}
	}
	return true;
}

/**
 * Print the mail form for visitors
 *
 */
function zcmail_printform() {
	$option = get_option('zcmail'); ?>
	<form action="" method="post"> <?php
	foreach($option['fields'] as $field => $values) {
		switch($values['type']) {
			case 'text':
			case 'email':
				zcmail_input( "zcmail[$field]", zcmail_get_value($values['label'], $_POST['zcmail'][$field]), zcmail_get_value_bool($values['label'], $_POST['zcmail'][$field]) );
				break;
			case 'textarea':
				zcmail_textarea("zcmail[$field]", zcmail_get_value($values['label'], $_POST['zcmail'][$field]), zcmail_get_value_bool($values['label'], $_POST['zcmail'][$field]) );
				break;
			case 'submit':
				zcmail_submit( $values['label'] );
				break;
		}
	} ?>
	</form> <?php
	if ($option['config']['homelink'] == 'Below the form' ) {
		zcmail_homelink();
	}
	elseif ($option['config']['homelink'] == 'In the footer') {
		add_action('wp_footer', 'zcmail_homelink');
	}
}



/*
 * Print the value the user submitted earlier, or the default.
 * Parameters: (str) default, (str) submitted post data
 * Return: (str) previous value
 */
function zcmail_get_value($default, $submitted) {
	if ($submitted) {
		return $submitted;
	}
	else {
		return $default;
	}
}

/*
 * Tell us if the value the user submitted earlier was the default value.
 * Parameters: (str) default, (str) submitted post data
 * Return: (str) previous value
 */
function zcmail_get_value_bool($default, $submitted) {
	if (!$submitted || $submitted == $default) {
		return true;
	}
	else {
		return false;
	}
}

/**
 * Print an input field.
 * @param string $name input field name
 * @param string $label label for the field
 * @param boolean $js show js, 'true' or 'false'
 * @param string $size size of the input field
 */
function zcmail_input( $name, $label, $js = true, $size=40 ) { ?>
	<input type="text" size="<?php echo $size ?>" name="<?php echo $name ?>" value="<?php echo $label ?>"<?php if ( $js ) zcmail_jsnippet( $label ); ?> class="zcmail_text" />
	<?php
}

/**
 * Print a textarea, hardcoded size so far
 * Parameters: (str) form input name, (str) lanel, (bool) use JavaScript
 */
function zcmail_textarea( $name, $label, $js ) { ?>
	<textarea rows="10" cols="50" name="<?php echo $name ?>" class="zcmail_textarea" <?php if ($js) zcmail_jsnippet($label) ?> ><?php echo $label; ?></textarea><?php
}

/**
 *  Print a submit putton
 *  Parameters: (str) button label
 */
function zcmail_submit( $label ) { ?>
	<input type="submit" id="submit" value="<?php echo $label ?>" class="zcmail_submit" /> <?php
}

/**
 * Print a select dropdown
 * Parameters: (str) form input name, (str) current value, (array) choices
 */
function zcmail_select( $name, $value, array $choices ) { ?>
	<select name="<?php echo $name ?>"><?php
	foreach ($choices as $choice) {
		if ($choice == $value ) { ?>
			<option value="<?php echo $choice ?>" selected><?php echo __($choice, 'zero-conf-mail') ?></option> <?php
		}
		else {?>
			<option value="<?php echo $choice ?>" ><?php echo __($choice, 'zero-conf-mail') ?></option> <?php
		}
	}
	?>
	</select>
	<?php
}

/**
 * Print a checkbox, checked or unchecked
 * Parameters: (str) from input name (bool) checked
 * TODO allow users to use this
 */
function zcmail_checkbox( $name, $true ) {
	if ($true == 'Enabled') { ?>
		<input type="checkbox" name="<?php echo $name ?>" value="checked" checked /> <?php
	}
	else { ?>
		<input type="checkbox" name="<?php echo $name ?>" value="checked" /> <?php
	}
}

/**
 * Print two radio select buttons for true/false choice
 * Parameters: (str) form input name, (str) true/false
 */
function zcmail_bool( $name, $true ) {
	if ( $true == 'true' ) { ?>
		<input type="radio" name="<?php echo $name ?>" value="true" checked />
		<input type="radio" name="<?php echo $name ?>" value="false" /> <?php
	}
	else { ?>
		<input type="radio" name="<?php echo $name ?>" value="true" />
		<input type="radio" name="<?php echo $name ?>" value="false" checked /> <?php
	}
}

/**
 * Print the javascript for input fields for the mail form for visitors
 * $param will be deleted or filled in
 * Prameters: (str) $param
 */
function zcmail_jsnippet($param) {
	echo " onfocus=\"if(this.value=='$param'){this.value=''};\" ";
	echo " onblur=\"if(this.value==''){this.value='$param'}\" ";
}

/**
 * The link to the plugin site.
 */
function zcmail_homelink() { ?>
	<span class="zcmail_home"><?php _e('Mail form powered by ', 'zero-conf-mail') ?><a href="<?php _e('http://www.nkuttler.de/wordpress/zero-conf-mail/', 'zero-conf-mail') ?>">Zero Conf Mail</a></span> <?php
}
